@@ -251,7 +251,9 @@ raschmodel <- function(y, weights = NULL, start = NULL, reltol = 1e-10,
   
     if(hessian) {
       vc <- if(deriv == "numeric") opt$hessian else ahessian(cf, esf)
-      vc <- qr.solve(vc)
+      ## FIXME: how to invert
+      vc <- chol2inv(chol(vc))
+      #vc <- qr.solve(vc)
     } else {
       vc <- matrix(NA, nrow = length(cf), ncol = length(cf))
     }
@@ -798,11 +800,8 @@ prm <- function(theta = NULL, beta = NULL)
 }
 
 ## rrm: calculate response matrices for given thetas and betas under the RM.
-rrm <- function(theta = NULL, beta = NULL, return_setting = TRUE)
+rrm <- function(theta, beta, return_setting = TRUE)
 {
-  ## check input
-  stopifnot(!is.null(theta) && !is.null(beta))
-  
   ## if list input, recurse...
   if (is.list(theta)) return(lapply(theta, rrm, beta = beta, return_setting = return_setting))
   if (is.list(beta)) return(lapply(beta, rrm, theta = theta, return_setting = return_setting))
@@ -814,5 +813,5 @@ rrm <- function(theta = NULL, beta = NULL, return_setting = TRUE)
   resp <- (matrix(runif(n * m), nrow = n, ncol = m) < probs) + 0
 
   ## return
-  if (return_setting) list(theta = theta, beta = beta, data = resp) else resp
+  if (return_setting) list(beta = beta, theta = theta, data = resp) else resp
 }
